<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xml:space="preserve">

  <xsl:output method="text" 
              indent="no" />

  <!-- We carry this in from outside -->
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.beanClass"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.implClass"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.package"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.stateful"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.jndiName"/>

  <!--Internally we use the variable instead -->
  <xsl:variable name="beanClass" select="$oracle.ds.v2.ejbgen.ejb.beanClass"/>
  <xsl:variable name="implClass" select="$oracle.ds.v2.ejbgen.ejb.implClass"/>
  <xsl:variable name="packageName" select="$oracle.ds.v2.ejbgen.ejb.package"/>
  <xsl:variable name="state" select="$oracle.ds.v2.ejbgen.ejb.stateful"/>
  <xsl:variable name="ejbName"><xsl:value-of select="$oracle.ds.v2.ejbgen.ejb.jndiName"/></xsl:variable>

  <xsl:template match="/">
    <xsl:apply-templates select="Service" />
  </xsl:template>

  <xsl:template match="Service">
/* Copyright (c) 2001, 2002, Oracle Corporation.  All rights reserved.  */

/*
   DESCRIPTION
    This file has been generated from the Oracle 9i Web Services.

   NOTES
    other useful comments, qualifications, etc.
*/

package <xsl:value-of select="$packageName"/>;

import java.io.*;
import java.util.*;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.CreateException;
import java.rmi.RemoteException;

import javax.naming.*;

import oracle.ds.v2.context.*;

public class <xsl:value-of select="$beanClass"/> extends <xsl:value-of select="$implClass"/>
  implements SessionBean
{
  // The session context
  SessionContext m_ctx;
  private Hashtable m_htEnv = null;

  // Methods of the SessionBean
  public void ejbCreate () throws RemoteException, CreateException { 
    if (m_htEnv == null) {
      try {
        m_htEnv = loadDsEnv();
      } catch (NamingException ne) {
        throw new RemoteException("env-entry lookup failed", ne);
      }
    }
    init(m_htEnv); 
  }  
  <xsl:choose>
  <xsl:when test="contains($state, 'stateless')">
  public void ejbRemove() {}
  public void ejbActivate () {}
  public void ejbPassivate () {}
  </xsl:when>
  <xsl:otherwise>
  public void ejbRemove() { close(); }
  public void ejbActivate () { activate(); }
  public void ejbPassivate () { passivate(); }
  </xsl:otherwise>
  </xsl:choose>
  // set it for both stateful and stateless 
  public void setSessionContext (SessionContext ctx) { m_ctx = ctx; }

  // load Ds Env. from system properties or env-entry
  private Hashtable loadDsEnv() throws NamingException {
    Hashtable ht = new Hashtable();
    
    // env from env-entries 
    Hashtable htEntries = loadDsEnvEntries();

    // try to load from System property first
    if (System.getProperty("http.proxyHost") != null) {
      ht.put(DsPropertyName.PROXY_HOST, System.getProperty("http.proxyHost"));
      ht.put(DsPropertyName.PROXY_PORT, System.getProperty("http.proxyPort"));
    }
    else {
      if (htEntries.get(DsPropertyName.PROXY_HOST) != null) {
         ht.put(DsPropertyName.PROXY_HOST, 
                (String) htEntries.get(DsPropertyName.PROXY_HOST));
         ht.put(DsPropertyName.PROXY_PORT,
                (String) htEntries.get(DsPropertyName.PROXY_PORT));
      }
    }
    if (System.getProperty("javax.net.ssl.KeyStore") != null) {
      ht.put(DsPropertyName.WALLET_LOC, 
             System.getProperty("javax.net.ssl.KeyStore"));
    }
    else {
      if (htEntries.get(DsPropertyName.WALLET_LOC) != null) {
        ht.put(DsPropertyName.WALLET_LOC, 
               (String) htEntries.get(DsPropertyName.WALLET_LOC));
      }
    }
         
    return ht;
  }

  /** get Ds Env from env-entry */
  private Hashtable loadDsEnvEntries() throws NamingException {
    Hashtable ht = new Hashtable();

    String szProxyHost = null;
    Integer iProxyPort = null;
    String szWalletLoc = null;

    Context   initCtx  = new InitialContext();
    Context   env      = (Context) initCtx.lookup("java:comp/env");
  
    try {
       szProxyHost = (String)  env.lookup("http.proxyHost");
       iProxyPort  = (Integer) env.lookup("http.proxyPort");
    } catch (NamingException e) {
       // Ignore it since these tow env-entries are optional
       // e.printStackTrace();
       // System.err.println("No proxy information is set in EJB <xsl:text>&lt;</xsl:text><xsl:value-of select="$ejbName"/><xsl:text>&gt;</xsl:text>");
    }

    if (szProxyHost != null <xsl:text>&amp;&amp;</xsl:text>
        szProxyHost.trim().length() <xsl:text>&gt; 0 &amp;&amp;</xsl:text>
        iProxyPort != null) {
      ht.put(DsPropertyName.PROXY_HOST, szProxyHost);
      ht.put(DsPropertyName.PROXY_PORT, iProxyPort.toString());
    }

    try {
       szWalletLoc = (String)  env.lookup("javax.net.ssl.KeyStore");
    } catch (NamingException e) {
       // Ignore it since this env-entry are optional
       // e.printStackTrace();
       // System.err.println("No Wallet information is set in EJB <xsl:text>&lt;</xsl:text><xsl:value-of select="$ejbName"/><xsl:text>&gt;</xsl:text>");
    }
    
    if (szWalletLoc != null <xsl:text>&amp;&amp;</xsl:text> szWalletLoc.trim().length() <xsl:text>&gt;</xsl:text> 0) {
      ht.put(DsPropertyName.WALLET_LOC, szWalletLoc);
    }

    return ht;  
  }       
} 
</xsl:template>

</xsl:stylesheet>
