<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  
  xml:space="preserve">

  <xsl:output method="text" indent="no"  />

  <!-- We carry this in from outside -->
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.clientClass"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.homeIf"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.jndiName"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.remoteIf"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.package"/>
  <xsl:param name="oracle.ds.v2.ejbgen.j2eeApplication"/>  

  <!-- Internally we use the variable instead -->
  <xsl:variable name="clientClass" select="$oracle.ds.v2.ejbgen.ejb.clientClass"/>
  <xsl:variable name="homeIfClass" select="$oracle.ds.v2.ejbgen.ejb.homeIf"/>
  <xsl:variable name="ejbName" select="$oracle.ds.v2.ejbgen.ejb.jndiName"/>
  <xsl:variable name="remoteIfClass" select="$oracle.ds.v2.ejbgen.ejb.remoteIf"/>
  <xsl:variable name="packageName" select="$oracle.ds.v2.ejbgen.ejb.package"/>
  <xsl:variable name="appName" select="$oracle.ds.v2.ejbgen.j2eeApplication"/>  

  <xsl:param name="index">0</xsl:param>   

  <xsl:template match="/">
    <xsl:apply-templates select="Service" />
  </xsl:template>

  <xsl:template match="Service">
/* Copyright (c) 2001, 2003, Oracle Corporation.  All rights reserved.  */

/*
   DESCRIPTION
    This file has been generated from the Oracle 9i Web Services.

   NOTES
    other useful comments, qualifications, etc.
*/
import java.io.*;
import java.util.*;
import javax.naming.*;

import org.w3c.dom.*;

import <xsl:value-of select="$packageName"/>.<xsl:value-of select="$remoteIfClass"/>;
import <xsl:value-of select="$packageName"/>.<xsl:value-of select="$homeIfClass"/>;

<xsl:apply-templates select="mapTypes" mode="importTypes"/>


/**
 * This is a simple client template. To compile it, 
 * please include the generated EJB jar file as well as
 * EJB and JNDI libraries in classpath.
 */
public class <xsl:value-of select="$clientClass"/>
{
  // replace the values
  private static String RMI_HOST  = "your j2ee server name";
  private static String RMI_PORT  = "your j2ee RMI port";
  private static String RMI_ADMIN = "your username";
  private static String RMI_PWD   = "your password";

  public <xsl:value-of select="$clientClass"/> () {}

  public static void main(String arg[]) {
    <xsl:value-of select="$clientClass"/> client = new <xsl:value-of select="$clientClass"/>();

  try {
    Hashtable env = new Hashtable();
    env.put(Context.INITIAL_CONTEXT_FACTORY, "com.evermind.server.rmi.RMIInitialContextFactory");
    env.put(Context.SECURITY_PRINCIPAL, RMI_ADMIN);
    env.put(Context.SECURITY_CREDENTIALS, RMI_PWD);
    env.put(Context.PROVIDER_URL, "ormi://" + RMI_HOST + ":" + RMI_PORT + "/<xsl:value-of select="$appName"/>");
    Context ctx = new InitialContext(env);
    <xsl:value-of select="$homeIfClass"/> home = (<xsl:value-of select="$homeIfClass"/>) ctx.lookup("<xsl:value-of select="$ejbName"/>");
      
    <xsl:value-of select="$remoteIfClass"/> service = home.create();

    // call any of the Remote methods below to access the EJB
    <xsl:apply-templates select="Operation"/>

    } catch (Throwable ex) {
       ex.printStackTrace();
    }
  }
} 
</xsl:template>


<!--
 |
 | Iterates over all the operations defined by the service
 |
 -->
<xsl:template match="Operation">
    <!--
       |
       | Variables used for the generation
       |
     -->    
    <xsl:variable name="returnType"><xsl:value-of select="@returnType"/></xsl:variable>
    <xsl:variable name="arrayType"><xsl:value-of select="@arrayType"/></xsl:variable>
    <xsl:variable name="opName"><xsl:value-of select="@name"/></xsl:variable>
    <xsl:variable name="methodName"><xsl:value-of select="@methodName"/></xsl:variable>
    <xsl:variable name="alignment"><xsl:value-of select="@alignSpaces"/></xsl:variable>
    <xsl:variable name="outputPartCount"><xsl:value-of select="count(Output/Part)"/></xsl:variable>
    <!--
       |
       | End of Variables used for the generation
       |
     -->
  <xsl:variable name="return"><xsl:value-of select="$returnType"/></xsl:variable>   
  <xsl:choose>
  <xsl:when test="$return='void'">
       //  service.<xsl:value-of select="$methodName"/>
       //  <xsl:text>(</xsl:text>
           <xsl:for-each select="Input/Part">
       //    <xsl:value-of select="concat(Class/@name, ' ')"/><xsl:value-of select="@name"/><xsl:if test="position() != last()"><xsl:text>, </xsl:text></xsl:if>
           </xsl:for-each>
       //  <xsl:text>)</xsl:text>;
  </xsl:when>
  <xsl:otherwise>
      // <xsl:value-of select="concat($return, ' ')"/>result_<xsl:value-of select="position()"/> = service.<xsl:value-of select="$methodName"/>
      // <xsl:text>(</xsl:text>
         <xsl:for-each select="Input/Part">
      //   <xsl:value-of select="Class/@name"/><xsl:text> </xsl:text><xsl:value-of select="@name"/><xsl:if test="position() != last()"><xsl:text>, </xsl:text></xsl:if>
         </xsl:for-each>
      // <xsl:text>//</xsl:text>;
  </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="mapTypes" mode="importTypes">
<xsl:apply-templates select="map" mode="importTypes"/>
</xsl:template>

<xsl:template match="map" mode="importTypes">
<xsl:if test="contains(@java-type,'.')">
import <xsl:value-of select='@java-type'/>;
</xsl:if>
</xsl:template>

</xsl:stylesheet>
