<xsl:stylesheet version="1.0"	xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output method="xml" 
              indent="yes" 
              omit-xml-declaration="no"
              doctype-public="-//Sun Microsystems, Inc.//DTD Enterprise JavaBeans 1.1//EN"
              doctype-system="http://java.sun.com/j2ee/dtds/ejb-jar_1_1.dtd"/>

  <!-- We carry this in from outside -->
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.jndiName"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.beanClass"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.homeIf"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.remoteIf"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.package"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.stateful"/>

  <xsl:variable name="packageName" select="$oracle.ds.v2.ejbgen.ejb.package"/>

  <xsl:template match="/">
    <xsl:apply-templates select="Service" />
  </xsl:template>

  <xsl:template match="Service">
    <!--
       |
       | Variables used for the generation
       |
     -->    
    <xsl:variable name="ejbName"><xsl:value-of select="$oracle.ds.v2.ejbgen.ejb.jndiName"/></xsl:variable>
    <xsl:variable name="dispName"><xsl:value-of select="Naming/Display"/></xsl:variable>
    <xsl:variable name="ejbDesc"><xsl:value-of select="Naming/Description"/></xsl:variable>
    <xsl:variable name="ejbClass"><xsl:value-of select="$packageName"/>.<xsl:value-of
select="$oracle.ds.v2.ejbgen.ejb.beanClass"/></xsl:variable>
    <xsl:variable name="ejbRemote"><xsl:value-of select="$packageName"/>.<xsl:value-of select="$oracle.ds.v2.ejbgen.ejb.remoteIf"/></xsl:variable>
    <xsl:variable name="ejbHome"><xsl:value-of select="$packageName"/>.<xsl:value-of select="$oracle.ds.v2.ejbgen.ejb.homeIf"/></xsl:variable>
    <xsl:variable name="state"><xsl:value-of select="$oracle.ds.v2.ejbgen.ejb.stateful"/></xsl:variable>
    <xsl:variable name="proxyHost"><xsl:value-of select="Proxy/@host"/></xsl:variable>
    <xsl:variable name="proxyPort"><xsl:value-of select="Proxy/@port"/></xsl:variable>
    <xsl:variable name="walletLoc"><xsl:value-of select="Wallet/@location"/></xsl:variable>

    <!--
       |
       | End of Variables used for the generation
       |
     -->
<ejb-jar>
   <enterprise-beans>
      <session>
         <description><xsl:value-of select="$ejbDesc"/></description>
         <display-name><xsl:value-of select="$dispName"/></display-name>
         <ejb-name><xsl:value-of select="$ejbName"/></ejb-name>
         <home><xsl:value-of select="$ejbHome"/></home>
         <remote><xsl:value-of select="$ejbRemote"/></remote>
         <ejb-class><xsl:value-of select="$ejbClass"/></ejb-class>
         <session-type><xsl:value-of select="$state"/></session-type>
         <transaction-type>Container</transaction-type>
         <xsl:if test="$proxyHost">
         <env-entry>
           <env-entry-name>http.proxyHost</env-entry-name>
           <env-entry-type>java.lang.String</env-entry-type>
           <env-entry-value><xsl:value-of select="$proxyHost"/></env-entry-value>
         </env-entry> 
         <env-entry>
           <env-entry-name>http.proxyPort</env-entry-name>
           <env-entry-type>java.lang.Integer</env-entry-type>
           <env-entry-value><xsl:value-of select="$proxyPort"/></env-entry-value>
         </env-entry> 
         </xsl:if>
         <xsl:if test="$walletLoc">
         <env-entry>
           <env-entry-name>javax.net.ssl.KeyStore</env-entry-name>
           <env-entry-type>java.lang.String</env-entry-type>
           <env-entry-value><xsl:value-of select="$walletLoc"/></env-entry-value>
         </env-entry> 
         </xsl:if>
      </session>
   </enterprise-beans>
</ejb-jar>

</xsl:template>

</xsl:stylesheet>
