<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xml:space="preserve">

  <xsl:output method="text" indent="no" />

  <!-- We carry this in from outside -->
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.implClass"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.package"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.serviceDescriptorRsrcPath"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.stateful"/>


  <!-- Internally we use the variable instead -->
  <xsl:variable name="implClass" select="$oracle.ds.v2.ejbgen.ejb.implClass"/>
  <xsl:variable name="packageName" select="$oracle.ds.v2.ejbgen.ejb.package"/>
  <xsl:variable name="serviceDescriptorRsrcPath" select="$oracle.ds.v2.ejbgen.ejb.serviceDescriptorRsrcPath"/>
  <xsl:variable name="state" select="$oracle.ds.v2.ejbgen.ejb.stateful"/>

  <xsl:template match="/">
    <xsl:apply-templates select="Service" />
  </xsl:template>

  <xsl:template match="Service">
    <!--
       |
       | Variables used for the generation
       | 
     -->    
    <xsl:variable name="rsrcPath"><xsl:value-of select="@path"/></xsl:variable>

    <!--
       |
       | End of Variables used for the generation
       |
     -->    
/* Copyright (c) 2001, 2002, Oracle Corporation.  All rights reserved.  */

/*
   DESCRIPTION
    This file has been generated by Oracle 9i Web Services.

   NOTES
    other useful comments, qualifications, etc.
*/

package <xsl:value-of select="$packageName"/>;

import java.io.*;
import java.util.*;
import java.rmi.RemoteException;
import org.w3c.dom.Element;

import oracle.ds.v2.*;
import oracle.ds.v2.proxy.*;
import oracle.ds.v2.impl.system.typemap.*;

<xsl:apply-templates select="mapTypes" mode="importTypes"/>

public class <xsl:value-of select="$implClass"/> implements java.io.Serializable 
{
  private DsWebServiceProxy m_proxy = null;

  <xsl:if test="contains($state, 'stateful')">
  // Hold the session identifier
  // clean the session 
  public void close() {
    m_proxy.close();
  }
  
  // make ds connection serializable
  protected void passivate() {
    m_proxy.passivate();
  }

  // This method is called to activate the session bean from second storage
  protected void activate() {    
    m_proxy.activate(getClass().getClassLoader());
  }
  </xsl:if>

  // This will be turned into the ejbCreate method
  // For now we just have a constructor
  public void init(Hashtable htEnv) throws RemoteException
  {
    try 
    {
      DsWebServiceProxyFactory proxyF = DsWebServiceProxyFactory.newInstance();
      <xsl:if test="contains($state, 'stateless')">
      m_proxy = proxyF.newDsWebServiceProxy(htEnv, 
                                            "<xsl:value-of select="$serviceDescriptorRsrcPath"/>",
                                            getClass().getClassLoader());
      </xsl:if>
      <xsl:if test="contains($state, 'stateful')">
      m_proxy = proxyF.newDsWebServiceProxy(htEnv, 
                                            "<xsl:value-of select="$serviceDescriptorRsrcPath"/>",
                                            getClass().getClassLoader(),
                                            true);  // maintain session
      </xsl:if>
      <xsl:apply-templates select="mapTypes"/>
    }    
    catch (DsException e) {
      e.printStackTraceWithNestedException();
      throw new RemoteException("Initialization of Web Service Failed: "+e.getMessage());
    }
  } // public init() throws RemoteException

  <!--
    |
    | Iterates over all the operations defined by the service
    |
   -->
  <xsl:apply-templates select="Operation"/>
}
</xsl:template>


<xsl:template match="mapTypes" mode="importTypes">
<xsl:apply-templates select="map" mode="importTypes"/>
</xsl:template>

<xsl:template match="map" mode="importTypes">
<xsl:if test="contains(@java-type,'.')">
import <xsl:value-of select='@java-type'/>;
</xsl:if>
</xsl:template>


<xsl:template match="mapTypes">
      //
      // Initialize the TypeMaping registry to support custom types
      //
      XmlJavaTypeMap typemap = null;<xsl:apply-templates select="map"/>
</xsl:template>

<xsl:template match="map">
      typemap = new XmlJavaTypeMap("<xsl:value-of select='@encodingStyle'/>",
                                   "<xsl:value-of select='@local-name'/>",
                                   "<xsl:value-of select='@namespace-uri'/>",
                                   "<xsl:value-of select='@java-type'/>",
                                   "<xsl:value-of select='@java2xml-class-name'/>",
                                   "<xsl:value-of select='@xml2java-class-name'/>",
                                   getClass().getClassLoader());
      m_proxy.addTypeMap(typemap);
</xsl:template>


<xsl:template match="Operation">

    <!--
       |
       | Variables used for the generation
       |
     -->    
    <xsl:variable name="returnType"><xsl:value-of select="@returnType"/></xsl:variable>
    <xsl:variable name="arrayType"><xsl:value-of select="@arrayType"/></xsl:variable>
    <xsl:variable name="opName"><xsl:value-of select="@name"/></xsl:variable>
    <xsl:variable name="opInputName"><xsl:value-of select="@input"/></xsl:variable>
    <xsl:variable name="opOutputName"><xsl:value-of select="@output"/></xsl:variable>
    <xsl:variable name="methodName"><xsl:value-of select="@methodName"/></xsl:variable>
    <xsl:variable name="alignment"><xsl:value-of select="@alignSpaces"/></xsl:variable>
    <xsl:variable name="inputPartCount"><xsl:value-of select="count(Input/Part)"/></xsl:variable>
    <xsl:variable name="outputPartCount"><xsl:value-of select="count(Output/Part)"/></xsl:variable>
    <!--
       |
       | End of Variables used for the generation
       |
     -->    

  public <xsl:value-of select="concat($returnType, ' ')"/> <xsl:value-of select="$methodName"/>
         <xsl:text>(</xsl:text>
         <xsl:for-each select="Input/Part">
           <xsl:value-of select="concat(Class/@name, ' ')"/> <xsl:value-of select="@name"/>
           <xsl:if test="position() != last()">
             <xsl:text>,
           </xsl:text><xsl:value-of select="$alignment"/>
           </xsl:if>
         </xsl:for-each>
         <xsl:text>) </xsl:text>
    throws RemoteException
  {
    try {

      // Build the array of part name and part values
      String szaPartNames[] = new String[<xsl:value-of select="$inputPartCount"/>];
      Object oaPartValues[] = new Object[<xsl:value-of select="$inputPartCount"/>];
      <xsl:apply-templates select="Input/Part" mode="input"/>
  
      // Perform the actual execution
      DsWebServiceMethod m = (DsWebServiceMethod) m_proxy.getMethod("<xsl:value-of select="$opName"/>", 
                                                                    "<xsl:value-of select="$opInputName"/>", 
                                                                    "<xsl:value-of select="$opOutputName"/>");
      Object objResp = m.invoke(szaPartNames, oaPartValues);

      <!--
        |
        | Handle the single output part case first
        |
      -->    
      <xsl:if test="$outputPartCount = 1">
      // Get the output from it
      return <xsl:apply-templates select="Output/Part" mode="casting"/>;
      </xsl:if>

      <!--
        |
        | Handle the multiple output part case first
        |
      -->    
      <xsl:if test="$outputPartCount > 1">
      // Get the output from it
      <xsl:value-of select="$returnType"/> retArray = new <xsl:value-of select="$arrayType"/>[<xsl:value-of select="$outputPartCount"/>];
      <xsl:apply-templates select="Output/Part" mode="output"/>
      return retArray;
      </xsl:if>
    } 
    catch (Exception e) {
      if (e instanceof DsException) {
       ((DsException) e).printStackTraceWithNestedException();
      }
      else {
       e.printStackTrace();
      }
      throw new RemoteException("Web Service Execution failed: "+e.getMessage());
    }

  }

</xsl:template>


<!--
 |
 | Part Input
 |
-->    
<xsl:template match="Part" mode="input">
    <!--
       |
       | Variables used for the generation
       |
     -->    
    <xsl:variable name="partCounter"><xsl:value-of select="position()-1"/></xsl:variable>
    <xsl:variable name="partVarName">reqPart<xsl:value-of select="$partCounter"/></xsl:variable>
    <xsl:variable name="partName"><xsl:value-of select="@name"/></xsl:variable>
    <xsl:variable name="partClass"><xsl:value-of select="Class"/></xsl:variable>
    <xsl:variable name="partPrimitive"><xsl:value-of select="Class/@isPrimitive"/></xsl:variable>
    <xsl:variable name="partObjectClass"><xsl:value-of select="Class/@objectClassName"/></xsl:variable>

    <xsl:variable name="partDeclaration"><xsl:if test="$partPrimitive = 'true'">new <xsl:value-of select="$partObjectClass"/>(<xsl:value-of select="$partName"/>)</xsl:if>
      <xsl:if test="$partPrimitive = 'false'"><xsl:value-of select="$partName"/></xsl:if>
    </xsl:variable>
    <!--
       |
       | End of Variables used for the generation
       |
     -->    
      //
      // Part #<xsl:value-of select="$partCounter"/> (<xsl:value-of select="$partName"/>)
      //
      szaPartNames[<xsl:value-of select="$partCounter"/>] = "<xsl:value-of select="$partName"/>";
      oaPartValues[<xsl:value-of select="$partCounter"/>] = <xsl:value-of select="$partDeclaration"/>;
</xsl:template>


<!--
 |
 | Part Output
 |
-->    
<xsl:template match="Part" mode="output">      
      retArray[<xsl:value-of select="position()-1"/>] = <xsl:if test="Class/@isPrimitive = 'true'">((<xsl:value-of select="Class/@objectClassName"/>) objResp[<xsl:value-of select="position()-1"/>]).<xsl:value-of select="Class/@name"/>Value()</xsl:if>
  <xsl:if test="Class/@isPrimitive = 'false'">(<xsl:value-of select="Class/@name"/>) objResp[<xsl:value-of select="position()-1"/>]</xsl:if>
</xsl:template>

<xsl:template match="Part" mode="casting">
  <xsl:if test="Class/@isPrimitive = 'true'">((<xsl:value-of select="Class/@objectClassName"/>) objResp).<xsl:value-of select="Class/@name"/>Value()</xsl:if>
  <xsl:if test="Class/@isPrimitive = 'false'">(<xsl:value-of select="Class/@name"/>) objResp</xsl:if>
</xsl:template>


</xsl:stylesheet>
