<xsl:stylesheet version="1.0"	
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xml:space="preserve">

  <xsl:output method="text" indent="no"/>

  <!-- We carry this in from outside -->
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.remoteIf"/>
  <xsl:param name="oracle.ds.v2.ejbgen.ejb.package"/>

  <!-- Internally we use the variable instead -->
  <xsl:variable name="remoteIfClass" select="$oracle.ds.v2.ejbgen.ejb.remoteIf"/>
  <xsl:variable name="packageName" select="$oracle.ds.v2.ejbgen.ejb.package"/>

  <xsl:template match="/">
    <xsl:apply-templates select="Service" />
  </xsl:template>

  <xsl:template match="Service">    
/* Copyright (c) 2001, Oracle Corporation.  All rights reserved.  */

/*
   DESCRIPTION
    This file has been generated by Oracle 9i Web Services.

   NOTES
    other useful comments, qualifications, etc.
*/

package <xsl:value-of select="$packageName"/>;

import javax.ejb.EJBObject;
import java.rmi.RemoteException;
<xsl:apply-templates select="mapTypes" mode="importTypes"/>

public interface <xsl:value-of select="$remoteIfClass"/> extends EJBObject
{
  <!--
    |
    | Iterates over all the operations defined by the service
    |
   -->
  <xsl:apply-templates select="Operation"/>
}
</xsl:template>

<xsl:template match="Operation">
    <!--
       |
       | Variables used for the generation per operation
       |
     -->    
    <xsl:variable name="returnType"><xsl:value-of select="@returnType"/></xsl:variable>
    <xsl:variable name="opName"><xsl:value-of select="@name"/></xsl:variable>
    <xsl:variable name="methodName"><xsl:value-of select="@methodName"/></xsl:variable>
    <xsl:variable name="alignment"><xsl:value-of select="@alignSpaces"/></xsl:variable>
    <xsl:variable name="outputPartCount"><xsl:value-of select="count(Output/Part)"/></xsl:variable>
    <!--
       |
       | End of Variables used for the generation
       |
     -->    
  public <xsl:value-of select="concat($returnType, ' ')"/>
         <xsl:value-of select="$methodName"/>
         <xsl:text>(</xsl:text>
         <xsl:for-each select="Input/Part">
           <xsl:value-of select="concat(Class/@name, ' ')"/>
           <xsl:value-of select="@name"/>
           <xsl:if test="position() != last()">
             <xsl:text>,
  </xsl:text><xsl:value-of select="$alignment"/>
           </xsl:if>
         </xsl:for-each>
         <xsl:text>) </xsl:text>
    throws RemoteException;
</xsl:template>


<xsl:template match="mapTypes" mode="importTypes">
<xsl:apply-templates select="map" mode="importTypes"/>
</xsl:template>

<xsl:template match="map" mode="importTypes">
<xsl:if test="contains(@java-type,'.')">
import <xsl:value-of select='@java-type'/>;
</xsl:if>
</xsl:template>

</xsl:stylesheet>
