<?zml version="1.0" ?>
<xsl:stylesheet version="1.0" 

                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
	  	          xmlns:sd="http://xmlns.oracle.com/wsb/2001/SERVICE_DESCRIPTOR"
                xmlns:dssoappa="http://www.oracle.com/ds/2000/SOAP_PA_PARAMS" 
                xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/">

  <xsl:output mode="xml" indent="yes"/>

  <!--
     |
     |
     | This stylesheet is used by WsdlDServicePkg and 
     | constructs a service descriptor from the wsdl document.
     | It requires at least the binding name and soap address location 
     | (or serviceName and portName) and portType.
     |
     |
   -->

  <!-- parameters passed by caller -->  
  <xsl:param name="serviceName"><xsl:value-of select="wsdl:definitions/wsdl:service[1]/@name"/></xsl:param>
  <xsl:param name="portName"><xsl:value-of select="wsdl:definitions/wsdl:service[@name=$serviceName]/wsdl:port[1]/@name"/></xsl:param>
  <xsl:param name="interfaceHref"></xsl:param>
  <xsl:param name="releaseDate"></xsl:param>
  <!-- bindingName may be passed along with the soap 
       address location, otherwise is
       retrieved from the wsdl using service name and port.--> 
  <xsl:param name="bindingName">
    <xsl:choose>
      <xsl:when test="contains(/wsdl:definitions/wsdl:service[@name=$serviceName]/wsdl:port[@name=$portName]/@binding,':')"> 
        <xsl:value-of select="substring-after(/wsdl:definitions/wsdl:service[@name=$serviceName]/wsdl:port[@name=$portName]/@binding,':')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="/wsdl:definitions/wsdl:service[@name=$serviceName]/wsdl:port[@name=$portName]/@binding"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:param>

  <xsl:param name="portType"></xsl:param>


  <!-- the soap:address element is rebuilt using the parameter that may be passed by
       caller -->
  <xsl:param name="soapAddressLoc">
    <xsl:value-of select="/wsdl:definitions/wsdl:service[@name=$serviceName]/wsdl:port[@name=$portName]/soap:address/@location"/>    
  </xsl:param>
  <xsl:variable name="soapAddress">
    <soap:address location="{$soapAddressLoc}"/>
  </xsl:variable>


  <!--
     |
     | Main template
     | Build constant stuff and external wrapper of the service descriptor
     |
   -->
  <xsl:template match="/wsdl:definitions"> 

    <xsl:variable name="interfacePortType" select="$portType"/>

      <sd:SERVICE_DESCRIPTOR xmlns:xlink="http://www.w3.org/1999/xlink">

        <sd:SERVICE_HEADER>
          <sd:NAMING>
            <sd:ID>urn:<xsl:value-of select="$serviceName"/></sd:ID> 
            <sd:NAME><xsl:value-of select="$serviceName"/></sd:NAME> 
            <sd:DESCRIPTION>-<xsl:value-of select="wsdl:service[@name=$serviceName]/wsdl:documentation"></xsl:value-of></sd:DESCRIPTION> 
          </sd:NAMING>
          <sd:PACKAGE> 
            <sd:VERSION>1.0</sd:VERSION> 
            <sd:RELEASEDATE><xsl:value-of select="$releaseDate"/></sd:RELEASEDATE> 
            <sd:UPDATEURL /> 
          </sd:PACKAGE>
          <sd:PROVIDER>
            <sd:CONTACTS/>
          </sd:PROVIDER>
          <sd:INTERFACE portType="{$interfacePortType}" xlink:href="{$interfaceHref}" /> 
        </sd:SERVICE_HEADER>
 
        <sd:SERVICE_DEPLOYMENT />

        <sd:SERVICE_RENDERERS /> 
  
        <xsl:apply-templates select="/wsdl:definitions/wsdl:binding[@name=$bindingName]"/>                                       

      </sd:SERVICE_DESCRIPTOR>

  </xsl:template>

  <!-- tempalte to parse  the binding -->
  <xsl:template match="wsdl:binding">
    <sd:SERVICE_BODY>
      <xsl:apply-templates select="wsdl:operation"/>
    </sd:SERVICE_BODY>
  </xsl:template>

  <!-- get operation info and build ds:operation -->
  <xsl:template match="wsdl:operation">
    <xsl:variable name="operationName" select="@name" />    
    <xsl:variable name="inputName" select="wsdl:input/@name"/>
    <xsl:variable name="outputName" select="wsdl:output/@name"/>
    <sd:OPERATION name="{$operationName}" input="{$inputName}" output="{$outputName}">
      <sd:PROTOCOL>
        <sd:ADAPTOR name="oracle.ds.v2.impl.adaptor.protocol.soap.DsSoapProtocolAdaptor">
          <xsl:call-template name="SERVICE_ADAPTOR">
            <xsl:with-param name="p_operationName" select="$operationName"/>
            <xsl:value-of select="."/>            
          </xsl:call-template>
        </sd:ADAPTOR>
      </sd:PROTOCOL>
    </sd:OPERATION>
  </xsl:template>

  <!-- template called to build the adaptor -->
  <xsl:template name="SERVICE_ADAPTOR">
    <xsl:param name="p_operationName"></xsl:param>
     <sd:PARAMETERS>
       <dssoappa:SOAP_PA_PARAMS >
         <dssoappa:Timeout>0</dssoappa:Timeout> 
         <xsl:copy-of select="$soapAddress"/>
         <xsl:copy-of select="/wsdl:definitions/wsdl:binding[@name=$bindingName]/soap:binding"/>
         <xsl:copy-of select="soap:operation"/>
         <xsl:apply-templates select="wsdl:input"/>
         <xsl:apply-templates select="wsdl:output"/>
         <xsl:apply-templates select="wsdl:fault"/>
         <!--
           wil be enable if parser handles namespaces. 
           <xsl:copy-of select="wsdl:input"/>
           <xsl:copy-of select="wsdl:output"/>
           <xsl:copy-of select="wsdl:fault"/>
         -->
       </dssoappa:SOAP_PA_PARAMS> 
     </sd:PARAMETERS>
  </xsl:template>

  <!-- 
       templates for wsdl input/output/fault element may be removed if copy-of
       elements is enabled (see above)
   -->
  <xsl:template match="/wsdl:definitions/wsdl:binding/wsdl:operation/wsdl:input" >
    <wsdl:input>
      <xsl:copy-of select="soap:body"/>
    </wsdl:input>
  </xsl:template>


  <xsl:template match="/wsdl:definitions/wsdl:binding/wsdl:operation/wsdl:output" >
    <wsdl:output>
      <xsl:copy-of select="soap:body"/>
    </wsdl:output>
  </xsl:template>


  <xsl:template match="/wsdl:definitions/wsdl:binding/wsdl:operation/wsdl:fault" >
    <wsdl:fault>
      <xsl:copy-of select="soap:body"/>
    </wsdl:fault>
  </xsl:template>


</xsl:stylesheet>

      







