------------------------------------------------------------------------------ 
-- 
--  @Copyright (c) 2002, Oracle Corporation. All rights reserved. 
--  @headcom 
-- 
--  This script seeds the wwwcp_obfuscation$ table with a obfuscation key
--  so that data obfuscation can be achieved using that key.
-- 
--  @usage   This should be run under the Portal schema.
-- 
--  @scope   Internal
-- 
-------------------------------------------------------------------------------

-- This block generates the seed key.
-- prompt Generating obfuscation key for encryption;
declare
    l_obfuscation_key raw(24);
    l_key_seed        raw(80);
    l_key_exists      integer;
begin
    -- Find out if we had a key before.
    select count(*) into l_key_exists
    from wwwcp_obfuscation$ o;

    -- If there is already a key don't do anything, because 
    -- old data would no longer be decipherable.
    if l_key_exists = 0 then
       
        begin
            -- Create the seed used to generate the key.
            l_key_seed := utl_raw.cast_to_raw('12345678901234567890123456789012345678901234567890123456789012345678901234567890');
        exception
            when OTHERS then
                dbms_output.put_line('ERROR: ' ||
                    'Can''t convert generator seed from varchar2 to RAW. ' || 
                    sqlerrm
                );
        end;
        
        begin
            -- Use DES3 to get the key.
            dbms_obfuscation_toolkit.des3getkey(
                which => 1, 
                seed  => l_key_seed,
                key   => l_obfuscation_key
            );
        exception           
           when others then
                dbms_output.put_line('ERROR: ' ||
                    'Can''t generate obfuscation key. ' || 
                    sqlerrm
                );
        end;
        
        begin
            -- Insert into the table if not present already.
            insert into wwwcp_obfuscation$ values ( l_obfuscation_key );
            
            -- Commit to be safe.
            commit;
        exception
           when others then
                dbms_output.put_line('ERROR: ' ||
                    'Can''t insert obfuscation key. ' || 
                    sqlerrm
                );
        end;
        -- Print a message saying we successfully created a key
        dbms_output.put_line('Created new obfuscation key!!!');
    else
        -- Print a message saying we preserved the old key because it exists
        dbms_output.put_line('Preserved old obfuscation key!!!');
    end if;
end;
/

